﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using Telerik.WinControls;
using Telerik.WinControls.UI;
using System.Data;

namespace TestApp
{
    public class ListBoxColumn : GridViewDataColumn
    {
        public ListBoxColumn(string name)
            : base(name)
        {
        }

        public override Type GetCellType(GridViewRowInfo row)
        {
            if (row is GridViewDataRowInfo)
            {
                return typeof(ListBoxCell);
            }
            return base.GetCellType(row);
        }
    }

    public class LinkItem : LightVisualElement
    {
        static Font linkFont = new Font("Segoe UI", 9f, FontStyle.Underline);

        public LinkItem(string text)
        {
            this.Text = text;
        }

        protected override void InitializeFields()
        {
            base.InitializeFields();

            this.Font = linkFont;
            this.ForeColor = Color.Blue;
            this.StretchHorizontally = false;
            this.StretchVertically = false;
            this.Alignment = ContentAlignment.MiddleRight;
        }
    }

    public class ListBoxCell : GridDataCellElement
    {
        RadListElement listElement;
        LinkItem linkDone;
        LinkItem linkRemove;
        StackLayoutElement linksContainer;
        RadDropDownListElement dropDownListElement;

        public ListBoxCell(GridViewColumn column, GridRowElement row)
            : base(column, row)
        {
        }

        protected override void CreateChildElements()
        {
            base.CreateChildElements();

            StackLayoutElement stack = new StackLayoutElement();
            stack.Orientation = Orientation.Vertical;
            stack.StretchHorizontally = true;
            stack.StretchVertically = true;

            this.Children.Add(stack);

            dropDownListElement = new RadDropDownListElement();
            dropDownListElement.Margin = new System.Windows.Forms.Padding(0, 0, 0, 4);
            dropDownListElement.Visibility = ElementVisibility.Collapsed;
            dropDownListElement.StretchVertically = false;

            dropDownListElement.ListElement.BackColor = Color.LightGray;
            dropDownListElement.ListElement.GradientStyle = GradientStyles.Solid;

            stack.Children.Add(dropDownListElement);

            DataTable table = new DataTable();
            table.Columns.Add("ID", typeof(int));
            table.Columns.Add("Name", typeof(string));
            table.Rows.Add(1, "one");
            table.Rows.Add(2, "two");
            table.Rows.Add(3, "three");
            dropDownListElement.DisplayMember = "Name";
            dropDownListElement.ValueMember = "ID";
            dropDownListElement.DataSource = table;

            listElement = new RadListElement();
            listElement.StretchVertically = true;
            listElement.StretchHorizontally = true;
            stack.Children.Add(listElement);

            linksContainer = new StackLayoutElement();
            linksContainer.StretchVertically = false;
            linksContainer.StretchHorizontally = true;
            linksContainer.Visibility = ElementVisibility.Collapsed;
            stack.Children.Add(linksContainer);

            linkDone = new LinkItem("Done");
            linksContainer.Children.Add(linkDone);

            LightVisualElement stretchElement = new LightVisualElement();
            stretchElement.StretchHorizontally = true;
            linksContainer.Children.Add(stretchElement);

            linkRemove = new LinkItem("Remove");
            linksContainer.Children.Add(linkRemove);
        }

        protected override Type ThemeEffectiveType
        {
            get
            {
                return typeof(GridDataCellElement);
            }
        }

        public override void UpdateInfo()
        {
            base.UpdateInfo();
            if (linksContainer != null)
            {
                linksContainer.Visibility = RowInfo.IsCurrent ? ElementVisibility.Visible : ElementVisibility.Collapsed;
                dropDownListElement.Visibility = RowInfo.IsCurrent ? ElementVisibility.Visible : ElementVisibility.Collapsed;
                this.Children[0].Visibility = RowInfo.IsCurrent ? ElementVisibility.Visible : ElementVisibility.Collapsed;
            }
        }

        protected override void SetContentCore(object value)
        {
            listElement.Items.Clear();
            if (value != null && value != DBNull.Value)
            {
                string[] phones = value.ToString().Split('\n');
                foreach (string phone in phones)
                {
                    listElement.Items.Add(new RadListDataItem(phone));
                }
            }
        }

        protected override SizeF MeasureOverride(SizeF availableSize)
        {
            if (linksContainer.Visibility == ElementVisibility.Visible)
            {
                listElement.MaxSize = new System.Drawing.Size(0, (int)availableSize.Height - 47);
            }
            else
            {
                listElement.MaxSize = Size.Empty;
            }
            return base.MeasureOverride(availableSize);
        }

        public override bool IsCompatible(GridViewColumn data, object context)
        {
            return data.Name == "Phones" && context is GridDataRowElement;
        }
    }
}
